/***************************************************************************************************/
/*                                                                                                 */
/* file:          Ringbuffer.h                                                                     */
/*                                                                                                 */
/* source:        2015-2106, written by Adrian Kundert                                             */
/*                                                                                                 */
/* description:   Simple ring buffer for a sequenced serial.print()                                */
/*                                                                                                 */
/***************************************************************************************************/

#ifndef RINGBUFFER_H
#define RINGBUFFER_H

class RingBuffer
{
public:
    RingBuffer() {
        Head = Tail = 0;
        BufferFull = false;
    }
    byte write(int i) {
        byte writenCnt=0;
        char str[16];
        itoa(i, str, 10);
        writenCnt = write(str);
        return writenCnt;
    }
    byte write(long i) {
        byte writenCnt=0;
        char str[16];
        ltoa(i, str, 10);
        writenCnt = write(str);
        return writenCnt;
    }
    byte write(float f, byte minStringWidthIncDecimalPoint, byte numVarsAfterDecimal) {
        byte writenCnt=0;
        const byte SIZE = 16;
        char str[SIZE];
        if ((minStringWidthIncDecimalPoint + numVarsAfterDecimal) < SIZE) { // ensure string length
          dtostrf(f, minStringWidthIncDecimalPoint, numVarsAfterDecimal, str);
          writenCnt = write(str);
        }
        return writenCnt;
    }    
    byte write(char* data) {
        byte i=0;
        while((data[i] != NULL) && (BufferFull == false)) {
            Buffer[Head++] = data[i++];
            if (Head >= BufferSize) Head = 0; //index reached buffer's end
            if (Head == Tail) BufferFull = true;
        }
        return i; // return the written count
    }
    bool getChar(char* data) {
        bool retVal = false;
        if ((Head != Tail) || (BufferFull == true)) {
            BufferFull = false; // is no more full after reading a char
            data[0] = Buffer[Tail++];
            if (Tail >= BufferSize) Tail = 0; //index reached buffer's end
            retVal = true; // a valid char
        }        
        return retVal;
    }

    static const byte BufferSize = 128;
private:
    byte Head;
    byte Tail;
    bool BufferFull;
    char Buffer[BufferSize];
};

#endif //RINGBUFFER_H


